// $Id$
//
// Copyright (C) 2008 

/** \file
    \brief main non-inline non-template implementation */

//#include "main.hh"
//#include "main.ih"

// Custom includes
#include <boost/algorithm/string/predicate.hpp>
#include <senf/Utils/Daemon.hh>
#include <senf/Utils/Logger/SenfLog.hh>
#include "UDPForwarder.hh"

//#include "main.mpp"
#define prefix_
///////////////////////////////cc.p////////////////////////////////////////

class UDPForwarderDaemon : public senf::Daemon
{
    SENF_LOG_CLASS_AREA();

    void configure();
    void main();

    static void terminate(siginfo_t const &);
};

prefix_ void UDPForwarderDaemon::configure()
{
    pidFile("udpforward.pid");
    consoleLog("udpforward.log");

    senf::Daemon::configure();
}

prefix_ void UDPForwarderDaemon::main()
{
    try {
        removeDaemonArgs();

        if (argc() != 3) {
            std::cerr << "Invalid number of arguments\n"
                      << "Usage: " << argv()[0] << " <listen port> <forward port> <target>\n";
            exit(1);
        }
        
        senf::log::ConsoleTarget::instance().route<senf::log::VERBOSE>();

        SENF_LOG(("udpforwarder starting"));
        
        senf::INet4SocketAddress listenPort (argv()[1]);
        senf::INet4SocketAddress targetPort (argv()[2]);

        udpforward::UDPForwarder forwarder ( listenPort, targetPort );

	senf::scheduler::SignalEvent sigintEvent (SIGINT, &terminate);
	senf::scheduler::SignalEvent sigtermEvent (SIGTERM, &terminate);

        detach();
        
        senf::scheduler::process();

        SENF_LOG(("signal received. Terminating."));
    }
    catch (std::exception & ex) {
        SENF_LOG((senf::log::FATAL)(ex.what() << "\n" "terminating unexpectedly."));
        exit(1);
    }
}

prefix_ void UDPForwarderDaemon::terminate(siginfo_t const &) 
{
    senf::scheduler::terminate();
}

SENF_DAEMON_MAIN(UDPForwarderDaemon);

///////////////////////////////cc.e////////////////////////////////////////
#undef prefix_
//#include "main.mpp"


// Local Variables:
// mode: c++
// fill-column: 100
// indent-tabs-mode: nil
// compile-command: "scons"
// comment-column: 40
// End:
